/*
  portasm.S - ASM implementation of routines defined in portable.h 

  Part of FreeRTOS NIOS2 port - Copyright (C) 2006 Kolja Waschk

  This port is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This port is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  A special exception to the GPL can be applied should you wish to distribute
  a combined work that includes this port, without being obliged to provide
  the source code for any proprietary components.  See the licensing section 
  of http://www.FreeRTOS.org for full details of how and when the exception
  can be applied.

  You should have received a copy of the GNU General Public License
  along with FreeRTOS.org; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "FreeRTOSconfig.h"
#include "system.h"

#ifdef ALT_EXCEPTION_STACK
#error TODO: Support for separated exception stack not yet implemented.
    .extern __alt_exception_stack_pointer;
#ifdef ALT_STACK_CHECK
    .extern __alt_exception_stack_limit;
#endif
#endif

	.extern _impure_ptr
	.extern pxCurrentTCB
	.extern uxCriticalNesting
	.extern vTaskSwitchContext
	.extern	_impure_ptr

	.global vPortYieldASM
	.global vPortContextRestore
	.global vPortLoadParameters

	/* Allow use of register r1 */	
	.set noat
	
vPortYieldASM:
    rdctl r6, status
    addi sp, sp, -52
    stw r16, 0(sp)
    stw r17, 4(sp)
    stw r18, 8(sp)
    stw r19, 12(sp)
    stw r20, 16(sp)
    stw r21, 20(sp)
    stw r22, 24(sp)
    stw r23, 28(sp)
    stw fp,  32(sp)
    /* Save return address */
    stw ra,  36(sp)
    /* Save status */
    stw r6,  40(sp)
    /* Save value of uxCriticalNesting to stack */
    ldw r16, %gprel(uxCriticalNesting)(gp)
    stw r16, 44(sp)
    /* Save value of Newlib's _impure_ptr to stack */
    ldw r16, %gprel(_impure_ptr)(gp)
    stw r16, 48(sp)
    /* Save top of stack address to pxCurrentTCB */
    ldw r16, %gprel(pxCurrentTCB)(gp)
    stw sp, 0(r16)
    /* restore r16 just to be sure ;) */
    ldw	r16, 0(sp)

    call vTaskSwitchContext

vPortContextRestore:
    /* Load top of stack address from pxCurrentTCB */
    ldw r16, %gprel(pxCurrentTCB)(gp)
    ldw sp, 0(r16)
    /* Load value of Newlib's _impure_ptr from stack */
    ldw r16, 48(sp)
    stw r16, %gprel(_impure_ptr)(gp)
    /* Load value of uxCriticalNesting from stack */
    ldw r16, 44(sp)
    stw r16, %gprel(uxCriticalNesting)(gp)
   
    ldw r16, 0(sp)
    ldw r17, 4(sp)
    ldw r18, 8(sp)
    ldw r19, 12(sp)
    ldw r20, 16(sp)
    ldw r21, 20(sp)
    ldw r22, 24(sp)
    ldw r23, 28(sp)
    ldw fp,  32(sp)
    
    /* Disable interrupts */
    wrctl status, r0
    /* Load ra and status for use after eret */
    ldw ea,  36(sp)
    ldw at,  40(sp)
    wrctl estatus, at
    addi sp, sp, 52
    eret
    
vPortLoadParameters:
    ldw     r4, 0(sp)
    ldw     fp, 4(sp)
    ldw     ra, 8(sp)
    addi    sp, sp, 12
    ret
